# Text styling options
TextStyles = ("bold", "underline")

# Color mapping for terminal colors
COLORS_PAIRS = {
    1: "white",
    2: "black",
    3: "blue",
    4: "red",
    5: "green",
    6: "yellow",
    7: "magenta",
}

# Default width for input elements
DEFAULT_INPUT_WIDTH = 15
