"""Constants used throughout the ADOM module."""

URL_BAR_INDEX = -2

COLORS_PAIRS = {
    1: "white",
    2: "black",
    3: "blue",
    4: "red",
    5: "green",
    6: "yellow",
    7: "magenta",
    8: "cyan",
}

COLORS_PAIRS_REVERSE = {
    "white": 1,
    "black": 2,
    "blue": 3,
    "red": 4,
    "green": 5,
    "yellow": 6,
    "magenta": 7,
    "cyan": 8,
}

elementTypes = ["cont", "text", "link", "input", "br"]

validFormats = ["m100"]
